package fr.paris.lutece.maven;

import java.io.File;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

import org.apache.maven.archiver.MavenArchiveConfiguration;
import org.apache.maven.archiver.MavenArchiver;
import org.apache.maven.artifact.ArtifactUtils;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.codehaus.plexus.archiver.jar.JarArchiver;
import org.codehaus.plexus.util.StringUtils;

/**
 * Builds a site's final WAR from a Lutece core artifact and a set of Lutece
 * plugin artifacts.<br/> Note that the Lutece dependencies (core and plugins)
 * will only be updated the first time the WAR is created. Subsequent calls to
 * this goal will only update the site's specific files.<br/> If you wish to
 * force webapp re-creation (for instance, if you changed the version of a
 * dependency), call the <code>clean</code> phase before this goal.
 * 
 * @goal site-assembly
 * @requiresDependencyResolution compile
 */
public class AssemblySiteMojo extends AbstractLuteceWebappMojo {

    private static final TimeZone UTC_TIME_ZONE = TimeZone.getTimeZone( "UTC" );
    private static final String SNAPSHOT_PATTERN = "SNAPSHOT";

    
	/**
	 * The output date format.
	 * 
	 * @parameter default-value="yyyyMMdd.HHmmss"
	 * @required
	 */
    private String utcTimestampPattern;

	/**
	 * The name of the generated WAR file.
	 * 
	 * @parameter expression="${project.build.finalName}"
	 * @required
	 */
	private String finalName;

	/**
	 * A temporary directory used to hold the exploded version of the webapp.
	 * 
	 * @parameter expression="${project.build.directory}/${project.build.finalName}"
	 * @required
	 */
	private File explodedDirectory;

	/**
	 * Whether creating the archive should be forced.
	 * 
	 * @parameter expression="${jar.forceCreation}" default-value="false"
	 */
	private boolean forceCreation;

	/**
	 * The maven archive configuration to use.
	 * 
	 * See <a
	 * href="http://maven.apache.org/ref/current/maven-archiver/apidocs/org/apache/maven/archiver/MavenArchiveConfiguration.html">the
	 * Javadocs for MavenArchiveConfiguration</a>.
	 * 
	 * @parameter
	 */
	private MavenArchiveConfiguration archive = new MavenArchiveConfiguration();

	/**
	 * The Jar archiver.
	 * 
	 * @parameter expression="${component.org.codehaus.plexus.archiver.Archiver#jar}"
	 * @required
	 */
	private JarArchiver jarArchiver;

	/**
	 * Executes the mojo on the current project.
	 * 
	 * @throws MojoExecutionException
	 *             if an error occurred while building the artifact.
	 */
	public void execute() throws MojoExecutionException, MojoFailureException {
		if (!LUTECE_SITE_PACKAGING.equals(project.getPackaging())) {
			throw new MojoExecutionException(
					"This goal can be invoked only on a "
							+ LUTECE_SITE_PACKAGING + " project.");
		} else {
			getLog().info(
					"Assembly-site " + project.getArtifact().getType()
							+ " artifact...");
			assemblySite();
		}
	}

	private void assemblySite() throws MojoExecutionException {

		// Explode the webapp in the temporary directory
		explodeWebapp(explodedDirectory);
		explodeConfigurationFiles(explodedDirectory);
		
		// put the timestamp in the assembly name
		if( ArtifactUtils.isSnapshot( project.getVersion() ) )
		{
	        DateFormat utcDateFormatter = new SimpleDateFormat( utcTimestampPattern );
			String newVersion = utcDateFormatter.format( new Date() );
			finalName = StringUtils.replace( finalName, SNAPSHOT_PATTERN, newVersion );
		}

		// Make a war from the exploded directory
		File warFile = new File(outputDirectory, finalName + ".war");

		MavenArchiver archiver = new MavenArchiver();
		archiver.setArchiver(jarArchiver);
		archiver.setOutputFile(warFile);
		archive.setForced(forceCreation);
		try {
			if (explodedDirectory.exists()) {
				archiver.getArchiver().addDirectory(
						explodedDirectory,
						PACKAGE_WEBAPP_INCLUDES,
						PACKAGE_WEBAPP_RESOURCES_EXCLUDES );
			}
			archiver.createArchive(project, archive);
		} catch (Exception e) {
			throw new MojoExecutionException("Error assembling WAR", e);
		}
	}
}
