package fr.paris.lutece.maven;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.apache.maven.artifact.Artifact;
import org.apache.maven.artifact.resolver.ArtifactNotFoundException;
import org.apache.maven.artifact.resolver.ArtifactResolutionException;
import org.apache.maven.artifact.resolver.ArtifactResolutionResult;
import org.apache.maven.artifact.resolver.DebugResolutionListener;
import org.apache.maven.artifact.resolver.DefaultArtifactCollector;
import org.apache.maven.artifact.resolver.ResolutionNode;
import org.apache.maven.artifact.resolver.WarningResolutionListener;
import org.apache.maven.artifact.resolver.filter.ArtifactFilter;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.project.MavenProject;

/**
 * Explodes a test webapp for a Lutece plugin or site project.<br/> Note that
 * the Lutece dependencies (core and plugins) will only be updated the first
 * time the exploded webapp is created. Subsequent calls to this goal will only
 * update the project's specific files :
 * <ul>
 * <li>for a plugin project : the plugin-specific webapp elements and classes.</li>
 * <li>for a site project : the site-specific webapp elements.</li>
 * </ul>
 * If you wish to force webapp re-creation (for instance, if you changed the
 * version of a dependency), call the <code>clean</code> phase before this
 * goal.
 * 
 * @goal exploded
 * @execute phase="process-classes"
 * @requiresDependencyResolution compile
 */
public class ExplodedMojo extends AbstractLuteceWebappMojo {
	
	/**
	 * The name of the servlet api artifact id.
	 */
	protected static final String SERVLET_API = "servlet-api";
	
	private static MavenProject lastProject;
	
	private static File multiProjectlocalConfDirectory;
	
	/**
	 * Executes the mojo on the current project.
	 * 
	 * @throws MojoExecutionException
	 *             if an error occured while exploding the webapp.
	 */
	public void execute() throws MojoExecutionException, MojoFailureException {
		if (!LUTECE_CORE_PACKAGING.equals(project.getPackaging())
				&& !LUTECE_PLUGIN_PACKAGING.equals(project.getPackaging())
				&& !LUTECE_SITE_PACKAGING.equals(project.getPackaging())
				&& !POM_PACKAGING.equals(project.getPackaging())) {
			throw new MojoExecutionException(
					"This goal can be invoked only on a "
							+ LUTECE_CORE_PACKAGING + " or "
							+ LUTECE_PLUGIN_PACKAGING + " or "
							+ LUTECE_SITE_PACKAGING + " project.");
		}
		if (POM_PACKAGING.equals( project.getPackaging(  ) ) && project.isExecutionRoot(  ) ) {
			
			getLog().warn("Only calling goal on modules");
			
			lastProject = (MavenProject) reactorProjects.get( reactorProjects.size(  ) -1 );
			multiProjectlocalConfDirectory = localConfDirectory;

		} 
		else {
			if (reactorProjects.size() > 1 && !project.isExecutionRoot()) {
				testWebappDirectory = getRootProjectBuildDirectory();
			}
			explodeWebapp(testWebappDirectory);
			explodeConfigurationFiles(testWebappDirectory);
			
			if ( lastProject != null && project.getId().equals( lastProject.getId(  ) ) )
			{
				getLog().info("------------------------------------------------------------------------");
				getLog().info("Building Lutece Multi Project");
				getLog().info("   explode local configuration, copy dependencies");
				getLog().info("------------------------------------------------------------------------");
				explodeMultiProjectUserConfigurationFiles( getRootProjectBuildDirectory(  ), multiProjectlocalConfDirectory );
				
				// create  directory WEB-INF/lib in lutece-multi-project
				File webinfLib = new File(testWebappDirectory, "WEB-INF/lib");
				webinfLib.mkdirs();
				
				// copy dependencies in directory WEB-INF/lib
				getLog().info("Copy dependencies into lutece-multi-project target directory");
				copyDependencies( webinfLib , doDependencyResolution(  ) );
			}
		}
	}
	
	/**
	 * Use to filter duplicate dependencies in multi project
	 * 
	 * @return a list of artifacts whith no duplicate entry
	 */
	private Set<Artifact> doDependencyResolution(  )
	{
		Set<Artifact> artifactsReturn = new HashSet<Artifact>();
		
		/*----------------init and Config-------------*/		
		// new instance of artifactCollector
		artifactCollector = new DefaultArtifactCollector( );
		
		// Collector Filter jar artifacts in scope 'compile' or 'runtime'
		ArtifactFilter thirdPartyFilter = new ArtifactFilter() {
			public boolean include(Artifact artifact) {
				return ( !LUTECE_CORE_TYPE.equals( artifact.getArtifactId(  ) ) &&
						!SERVLET_API.equals( artifact.getArtifactId(  ) ) &&
						!Artifact.SCOPE_PROVIDED.equals(artifact.getScope() ) &&
						!Artifact.SCOPE_TEST.equals(artifact.getScope() ) );
			}
		};
		
		// Collector listener config
        List listeners = new ArrayList();        
        if (logger.isDebugEnabled()) {
            listeners.add(new DebugResolutionListener(logger));
        }
        listeners.add(new WarningResolutionListener(logger));
		
        /*---------------- Resolution-------------*/
		// resolve conflict version artifacts with collector
		ArtifactResolutionResult artifactResolutionResult = null;
		try 
		{
			artifactResolutionResult = artifactCollector.collect(multiProjectArtifacts, project.getArtifact(), localRepository, remoteRepositories, metadataSource, thirdPartyFilter, listeners);
		} 
		catch (ArtifactResolutionException e) 
		{
			e.printStackTrace();
		} 

		// keep track of added reactor projects in order to avoid duplicates
        Set emittedReactorProjectId = new HashSet();

        for (ResolutionNode node : (Set<ResolutionNode>)artifactResolutionResult.getArtifactResolutionNodes() ) 
        {
            Artifact art = node.getArtifact();
            
            try 
            {
            	resolver.resolve(art, node.getRemoteRepositories(), localRepository);
            }
            catch (ArtifactNotFoundException e) 
            {
            	e.printStackTrace();
            }
            catch (ArtifactResolutionException e) 
            {
            	e.printStackTrace();
            }
 
            if ( emittedReactorProjectId.add(art.getGroupId() + '-' + art.getArtifactId())) 
            {
                artifactsReturn.add(art);
            }
        }
        
        return artifactsReturn;
	}
	
	/**
	 * Copy the jar dependencies list iinto the output directory
	 * 
	 * @param outputDirectory the output directory
	 * @param listArtifacts the list of artifact dependency
	 * @throws MojoExecutionException the mojo exection exection
	 */
	public void copyDependencies( File outputDirectory, Set<Artifact> listArtifacts  ) throws MojoExecutionException
	{
		for (Artifact d : listArtifacts) 
		{
			File jarFile = d.getFile();
			File newFile = new File(outputDirectory, jarFile.getName());

			try 
			{
				FileUtils.copyFileIfModified(jarFile, newFile);						
			} 
			catch (IOException e)
			{
				throw new MojoExecutionException("Error while copying "
						+ jarFile.getAbsolutePath() + " to "
						+ newFile.getAbsolutePath(), e);
			}
		}
	}
	
	
	/**
	 * Explode multi project local configuration directory
	 * 
	 * @param targetDir the multi project target directory
	 * @param confDirectory The multi project local conf directory
	 * @throws MojoExecutionException the mojo exception
	 */
	protected void explodeMultiProjectUserConfigurationFiles(File targetDir, File confDirectory ) throws MojoExecutionException 
	{
		try {

			// Copy user-specific configuration files
			getLog().info(
					"Local configuration directory is "
							+ confDirectory.getAbsolutePath());
			if (confDirectory.exists()) {
				getLog().debug("Copying local configuration");
				FileUtils.copyDirectoryStructure(confDirectory, targetDir);
			} else {
				getLog().warn(
						"Local configuration directory "
								+ confDirectory.getAbsolutePath()
								+ " does not exist");
			}
		} catch (IOException e) {
			// Use the same catch block for all IOExceptions, presumably the
			// exception's message will be clear enough.
			throw new MojoExecutionException("Error while copying configuration resources", e);
		}
	}
}
